// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_WIN32_WAITER_H
#define HU_VARADIISTVAN_SCPL_IO_WIN32_WAITER_H
//------------------------------------------------------------------------------

#include "../Failable.h"

#include <set>

#include <cassert>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

class Event;

/**
 * Class that can be used to register OVERLAPPED structures and then
 * wait for their events.
 */
class Waiter : public Failable
{
private:
    /**
     * Type for the mapping of events to overlapped objects.
     */
    typedef std::set<Event*> events_t;

    /**
     * The events to be handled.
     */
    events_t events;

    /**
     * The number of events.
     */
    size_t numEvents;

public:
    /**
     * Construct the waiter.
     */
    Waiter();

    /**
     * Destroy the waiter. It will be removed from any Event
     * objects it may be related to.
     */
    ~Waiter();

    /**
     * Wait for an event to occur with the given timeout.
     *
     * @return true if any of the events belonging to the Overlapped
     * objects become ready, false if the timeout occurs.
     */
    bool wait(int timeout = -1);

protected:
    /**
     * Add the given event to the set of those to be waited for.
     */
    void add(Event* event);

    /**
     * Remove the given event from the set of those to be waited for.
     */
    void remove(Event* event);

    friend class Event;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline Waiter::Waiter() :
    numEvents(0)
{
}

//------------------------------------------------------------------------------

inline void Waiter::add(Event* event)
{
    events.insert(event);
    ++numEvents;
}

//------------------------------------------------------------------------------

inline void Waiter::remove(Event* event)
{
    assert(numEvents>0);
    --numEvents;
    events.erase(event);
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_WIN32_WAITER_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
